;
; FIR filter test
; Juha Niinikoski 27.05.2005
; 
; Modulo bug found, X and Y data spaces swapped 29.05.2005
; Hitec SAC.R bug corrected 30.05.2005
; Modified for two chanels 30.05.2005
;
#include <as30f6012.h>	; Processor EQUs, Compile with -P option.
;#include <asdspic.h>	This does not work !
;

;
; Y-data, be carefull with alignment, counted at bytes
; Modulo addressing needs zero boundary addresses
;
; Left CH
	psect	yfir,class=YDATA,space=1,reloc=256,pad=256

	global	_firl_taps		; tap vector
	global	_firl_taps_end	; last address of tap vector
	global	_firl_n			; filter length (N)

_firl_taps:		ds	256		; Filter coeffisients (bytes)
_firl_taps_end:	ds	2		; Last tap
_firl_n:			ds	2		; Filter length

; Right CH
	psect	yfir,class=YDATA,space=1,reloc=256,pad=256

	global	_firr_taps		; tap vector
	global	_firr_taps_end	; last address of tap vector
	global	_firr_n			; filter length (N)

_firr_taps:		ds	256		; Filter coeffisients (bytes)
_firr_taps_end:	ds	2		; Last tap
_firr_n:			ds	2		; Filter length


;
; X-data, be carefull with alignment
; Modulo addressing needs zero boundary addresses
;
; Left CH
	psect	xfir,class=XDATA,space=1,reloc=256,pad=256

	global	_firl_dly		; filter delay line
	global	_firl_dly_end	; last address of delay line
	global	_firl_dly_ptr	; insert pointer to delay line
	

_firl_dly:		ds	256		; filter delay line (bytes)
_firl_dly_end:	ds	2		; last address of delay line
_firl_dly_ptr:	ds	2		; delay buffer input pointer

;Right CH
	psect	xfir,class=XDATA,space=1,reloc=256,pad=256

	global	_firr_dly		; filter delay line
	global	_firr_dly_end	; last address of delay line
	global	_firr_dly_ptr	; insert pointer to delay line
	

_firr_dly:		ds	256		; filter delay line (bytes)
_firr_dly_end:	ds	2		; last address of delay line
_firr_dly_ptr:	ds	2		; delay buffer input pointer

;
; Filter code
;
	psect	text,class=CODE,delta=2,reloc=4
	global	_fir

;
; Dual Channell Single sample FIR filter
; Input, RXBUF0 & RXBUF1 = sample (n)
; Output, TXBUF0 & TXBUF1 = filter output (n)
;
; Tap vectors in Y-memory
; Delay (ring)buffer in X-memory
;

_fir:
; Save critical registers

        push    MODCON                          ; save context of MODCON
        push    XMODSRT                         ; save context of YMODSRT
        push    XMODEND                         ; save context of YMODEND
        push    CORCON                          ; save context of CORCON
        push    PSVPAG                          ; save context of PSVPAG
        push    w8                              ; save context of w8
        push    w10                             ; save context of w10
;
; Used, but not saved: W0, W1, W5, W6
;
; Set core mode for DSP calculations
;
        mov     #0x00B0,w8                      ; Enable Accumulator A Saturation and
                                                ; Data Space write Saturation
                                                ; as bits 5 and 7 are set in CORCON
        mov     w8, CORCON                      ; set PSV and saturation options		
;
; Set modulo addressing & delay buffer address
; Note W10 or W11 are only valid Y-memory modulo pointers
;
        mov     #0x8FF8, w8                    ; set XMD = 8 and YMD = no
        mov     w8, MODCON                     ; enable Y Modulus, next instruction
												; can not be indirect addressed 


; Left Channell filter
;
		mov		RXBUF0, w0						; get new Left CH sample
	
        mov     #_firl_dly,w8					; get ring buffer location
        mov     w8, XMODSRT                    ; YMODSRT = base address of delay line

		mov		_firl_dly_end, w8				; load last member address
        mov     w8, XMODEND                    ; YMODEND = end address of delay line

		mov		_firl_n, w1						; Number of taps
		dec		w1, w1							; W1 = Number of taps - 1
 
;
; Insert new sample to ring buffer @ X-memory space
; Modulo addressing works only in X-memory with non MAC class instructions
;
		mov _firl_dly_ptr, w8					; get delay line pointer
		mov w0, [w8++]							; store new sample (w0)
		mov	w8, _firl_dly_ptr					; w10 point oldest sample

;
; Tap vector @ Y-memory
;
		mov _firl_taps_end, w10					; tap vector end to w10

;
; Calculate Filter
; W10 point to tap vector last member @ Y-memory
; tap vector is read backwards
; W8 point to oldest sample at delay line @ X-memory
; sample buffer is read forward direction
;
		clr    A, [w8], w5, [w10], w6			; clear ACCA & get last tap & oldest sample
		repeat w1								; repeat count (W1) = Number of taps - 1
		mac w5*w6, A,  [w8]+=2, w5,  [w10]-=2, w6

;
; round and store result in AccA to output register
;
 
       sac.r   a,#1,w0
;
; Hitec v9.50 assembler generates garbage from sac.r instruction
; Should generate 0xCD0080 but generates 0xCD0001
; Patch to fix the problem 29.05.2005
;		sftac	a, #1
;		sac.r	a, w0
;
		mov		w0, TXBUF0;						; Left output
;

; Right Channell filter
;
		mov		RXBUF1, w0						; get new Right CH sample
	
        mov     #_firr_dly,w8					; get ring buffer location
        mov     w8, XMODSRT                    ; YMODSRT = base address of delay line

		mov		_firr_dly_end, w8				; load last member address
        mov     w8, XMODEND                    ; YMODEND = end address of delay line

		mov		_firr_n, w1						; Number of taps
		dec		w1, w1							; W1 = Number of taps - 1 
;
; Insert new sample to ring buffer @ X-memory space
; Modulo addressing works only in X-memory with non MAC class instructions
;
		mov _firr_dly_ptr, w8					; get delay line pointer
		mov w0, [w8++]							; store new sample (w0)
		mov	w8, _firr_dly_ptr					; w10 point oldest sample

;
; Tap vector @ Y-memory
;
		mov _firr_taps_end, w10					; tap vector end to w10

;
; Calculate Filter
; W10 point to tap vector last member @ Y-memory
; tap vector is read backwards
; W8 point to oldest sample at delay line @ X-memory
; sample buffer is read forward direction
;
		clr    A, [w8], w5, [w10], w6			; clear ACCA & get last tap & oldest sample
		repeat w1								; repeat count (W1) = Number of taps - 1
		mac w5*w6, A,  [w8]+=2, w5,  [w10]-=2, w6

;
; round and store result in AccA to output register
;
 
       sac.r   a,#1,w0
;
; Hitec v9.50 assembler generates garbage from sac.r instruction
; Should generate 0xCD0080 but generates 0xCD0001
; Patch to fix the problem 29.05.2005
;		sftac	a, #1
;		sac.r	a, w0
;
		mov		w0, TXBUF1;						; Right output
;	


;
; Restore critical registers
;
        clr     MODCON                          ; disable modulo addressing
        nop
        pop     w10                             ; restore context of w10
        pop     w8                              ; restore context of w8
        pop     PSVPAG                          ; restore context of PSVPAG
        pop     CORCON                          ; restore context of CORCON
        pop     XMODEND                         ; restore context of YMODEND
        pop     XMODSRT                         ; restore context of YMODSRT
        pop     MODCON                          ; restore context of MODCON
        nop
;
		return
