//
// PCM3002 Codec test
// Juha Niinikoski 18.05.2005
//
// Hi-Tech dsPICC_9.50 compiler, MPLAB 7.00
// Microchip dsPICDEM Starter Demo Board
//
// Xtal 4MHz / PLL 8x = 32MHz clock
//
// PCM3002 driver 20.05.2005
// FIR filter test added 30.05.2005
// Xtal 4MHz / PLL 16x = 64MHz clock = 16 Mips


// global variabbles
unsigned int vol;				// DAC attenuation bits

int sin_table[100];				// wave table for test
int sin_idx;
int cos_idx;
float phase;
bit mode;
bit iq_test;

// test
// [0] = low, [1] = high word
	union
		{
		long lo;
		int in[2];
		} test;

	float f;
	long l;

// include programs
#include <dspic.h>
#include "uart.c"
#include "pcm3002spi.c"
#include "math.h"

// processor config
//__CONFIG(FOSC, 0xFFFFF0 & POSC & CLKSWDIS); // XTL, error in old header file
//__CONFIG(FOSC, XTPLL8 & POSC & CLKSWDIS); // 4MHz*8=32MHz clock, 8MHz cycle
__CONFIG(FOSC, XTPLL16 & POSC & CLKSWDIS); // 4MHz*16=64MHz clock, 16MHz cycle
__CONFIG(FWDT, WDTDIS);
__CONFIG(FBORPOR, BOREN & BORV27 & MCLREN);
__CONFIG(FGS, GCPU & GWRU);

// test FIR filter stuff



// fir filter variables

// Left
extern int firl_taps[128];		// filter taps
extern int firl_taps_end;		// last members address
extern int firl_n;				// N, Number of taps

extern int firl_dly[128];		// filter delay line
extern int firl_dly_end;			// last address of ring buffer
extern int firl_dly_ptr;			// ring buffer insert pointer

// Right
extern int firr_taps[128];		// filter taps
extern int firr_taps_end;		// last members address
extern int firr_n;				// N, Number of taps

extern int firr_dly[128];		// filter delay line
extern int firr_dly_end;			// last address of ring buffer
extern int firr_dly_ptr;			// ring buffer insert pointer


// 1000Hz / 32 tap test LP filter
// 24kHz sampling rate

#define N 32
static int lp_taps[N] = {
0xFD1E,
0xFFB8,
0xFFFD,
0x0079,
0x012E,
0x021B,
0x033B,
0x0486,
0x05F0,
0x0768,
0x08DB,
0x0A37,
0x0B69,
0x0C60,
0x0D0D,
0x0D66,
0x0D66,
0x0D0D,
0x0C60,
0x0B69,
0x0A37,
0x08DB,
0x0768,
0x05F0,
0x0486,
0x033B,
0x021B,
0x012E,
0x0079,
0xFFFD,
0xFFB8,
0xFD1E
};

// 1000Hz / 32 tap test HP filter
// 24kHz sampling rate

static int hp_taps[N] = {
0xF05F,
0x0C85,
0x086E,
0x0582,
0x0341,
0x0150,
0xFF78,
0xFD98,
0xFBAA,
0xF9A7,
0xF7BA,
0xF5E6,
0xF458,
0xF328,
0xF25F,
0x7218,
0xF25F,
0xF328,
0xF458,
0xF5E6,
0xF7BA,
0xF9A7,
0xFBAA,
0xFD98,
0xFF78,
0x0150,
0x0341,
0x0582,
0x086E,
0x0C85,
0xF05F,
0x0000,
};


// init FIR filters

void init_fir(void)
	{
	int x;

// Left
	firl_n = N;
	firl_dly_ptr = (int)firl_dly;
	firl_dly_end = firl_dly_ptr + ((2*N)-1);		// end address(last byte) for modulo ring

	firl_taps_end = ((int)firl_taps) + ((2*N)-2);	// end address(last word) for tap vector

// Right
	firr_n = N;
	firr_dly_ptr = (int)firr_dly;
	firr_dly_end = firr_dly_ptr + ((2*N)-1);		// end address(last byte) for modulo ring

	firr_taps_end = ((int)firr_taps) + ((2*N)-2);	// end address(last word) for tap vector

	for(x=0; x<N; x++)
		{
		firl_dly[x] = 0;
		firl_taps[x] = lp_taps[x];

		firr_dly[x] = 0;
		firr_taps[x] = lp_taps[x];
		}
	}


// main
void main(void)
	{
	int x;
	unsigned char c;

// set general I/O
	TRISD4 = 0;				// LEDs = output
	TRISD5 = 0;
	TRISD6 = 0;
	TRISD7 = 0;

// init UART
	InitUSART1();				// wake up serial interface
	printf("\n\rdsPIC30F6012 / PCM3002 / first FIR test 30.05.2005 / JNi\n\r");
	printf("Xtal 4MHz / PLL 16x = 64MHz clock = 16 Mips, Codec sampling rate = 24kHz\n\r");

// Init Sin table for 100 samples, 240 Hz, fs = 24 kHz
	sin_idx = 0;
	cos_idx = 25;				// +90deg

#define pstep 0.062831852		// 2pi/100
	phase = 0;
	for(x=0; x<100; x++)
		{
		sin_table[x] = (sin(phase)) * 32767;
		phase = phase + pstep;
		}
	printf("Sine table initialized\n\r");

// Init FIR filter
	init_fir();
	printf("%d - tap test filter initialized\n\r", N);

// init soft SPI for PCM3002 control traffic
	init_spi();
	C_RESET = 1;			// start codec	
	for(x=0; x<32000; x++)	// codec start delay
		{
		asm("nop");
		}
// init PCM3002 codec
	init_pcm3002();			// init codec
	printf("PCM3002 Codec initialized\n\r");

// init dsPIC30F6012 DCI interface
	init_dci();				// init DCI

// enable IRQs
	DCIIF = 0;
	DCIIE = 1;				// enable DCI IRQ
	printf("DCI & IRQ system initialized\n\r");

// start forever loop
	RD4 = 1;				// run lamp
	for(;;)
		{

		if(kbhit())
			{
			c = getch();
			switch(c)
				{
				case 'l':
					printf("1000Hz Low pass selected\n\r");
					for(x=0; x<N; x++)
						{
						firl_taps[x] = lp_taps[x];
						firr_taps[x] = lp_taps[x];
						}
					mode = 1;
					break;

				case 'h':
					printf("1000Hz High pass selected\n\r");
					for(x=0; x<N; x++)
						{
						firl_taps[x] = hp_taps[x];
						firr_taps[x] = hp_taps[x];
						}
					mode = 1;
					break;

				case 'm':
					printf("Mixed selected. L=LP, R=HP\n\r");
					for(x=0; x<N; x++)
						{
						firl_taps[x] = lp_taps[x];
						firr_taps[x] = hp_taps[x];
						}
					mode = 1;
					break;

				case 'i':
					printf("240Hz I/Q test signal\n\r");
					iq_test = 1;
					mode = 1;
					break;

				case 'x': case 0x1B: case ' ':
					printf("Straight copy in to out\n\r");
					mode = 0;
					iq_test = 0;
					break;

				default:
					printf("Key is: %d\n\r", c);
					printf("Try: l, h, m, i or x\n\r");
				}
			}


#if 0
// Push Button logic, Volume control

		if(RC14 == 0 || RC13 == 0)
			{
			if(RC14 == 0)		// increase
				vol++;

			if(RC13 == 0)		// decrease
				vol--;

			vol = vol & 0xFF;
			write_spi((vol | LDR));	// write PCM3002 R0

			printf("DAC attenuation %d\n\r", vol);	
			for(x=0; x<30000; x++)	//  delay
				{
				asm("nop");
				}
			}
#endif


// output mode select
			if(RC14 == 0)
				mode = 1;

			if(RC13 == 0)
				mode = 0;;

			RD4 = mode;

		}	// end for(;;)
	}		// end main
